module Lifen
  class Client

    def request(mode, url, params = {})
      before_request

      response = faraday_client.send(mode) do |req|
        req.url url

        req.headers['Authorization']      = "Bearer #{bearer}"
        req.headers['Accept']             =  use_and_remove_accept(params)

        if mode == :post
          req.headers['Content-Type']     = "application/json"
        end

        req.body = JSON.generate(params)
      end

      handle_errors(response, params)

      handle_response(response)
    end

    def post(url, params = {})
      request(:post, url, params)
    end

    def put(url, params = {})
      request(:put, url, params)
    end

    def get(url, params = {})
      request(:get, url, params)
    end

    private

      def handle_errors(response, params)
        if response.status == 500

          json = JSON.parse response.body

          trace_id = json.fetch("X-B3-TraceId", "unknown")

          raise Error, "Error 500, Internal server error (trace ID: #{trace_id}), #{response_error(response, params)}"
        end
      end

      def handle_response(response)
        if response.headers['Content-Type'].match "json"
          JSON.parse response.body
        else
          response.body
        end
      end

      def faraday_client
        @faraday_client ||= Faraday.new(faraday_options) do |faraday|
          faraday.adapter  Faraday.default_adapter  # make requests with Net::HTTP
        end
      end

      def faraday_options
        options = {url: site}

        options[:proxy] = proxy_url if !proxy_url.nil?

        options
      end

      def site
        Lifen.configuration.site
      end

      def proxy_url
        Lifen.configuration.proxy_url
      end

      def before_request
      end

      def response_error(response, params)
        params[:payload] = "filtered" if params.is_a?(Hash) and params.has_key? :payload

        "#{response.env.method.upcase} '#{response.env.url}' with params '#{params.inspect}' and bearer '#{trucanted_bearer}'"
      end

      def trucanted_bearer
        if m = /^(.{24})(.*)$/.match(bearer)
          "#{m[1]}#{"*" * m[2].length}"
        else
          bearer
        end
      end

      def bearer
        raise "A bearer method must be defined"
      end

      def use_and_remove_accept(params)
        return params.delete :accept if params.is_a?(Hash) and params.has_key? :accept

        "application/json"
      end
  end
end
